package ocaml-protoc

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Pb_typing_resolution.Types_by_scopeSource

Custom container for all the types (message or enums) which are organized by their scope. This allow efficient search of a type given its type path

The construction of this container is the first step in the type resolution.

Sourcetype t

container type

Sourceval empty : t

empty container

add a protobuf type

find a protobuf type given its type path

Sourceval print : t -> unit

pretty print of the container